#! /usr/bin/env ipython
import numpy as np
import sys

import EPOS

'''
This example shows how to use EPOS with a planet formation/population synthesis model.

pfm is a dictionary with planet properties as numpy arrays. 
Planets with the same starID are assumed to be in a multi-planet systems
sma:	semi-major axis
mass:	planet mass
radius:	planet radius, optional if using a mass-radius relation
starID:	stellar identifier
inc:	planet mutual inclination
tag:	tag for simulation initial conditions such as Fe/H, optional
'''

''' initialize the EPOS class '''
epos= EPOS.classes.epos(name='example_3')

''' Load the Kepler planet list and detection efficiency '''
obs, survey= EPOS.kepler.dr25(Gaia=True, Vetting=True)
epos.set_observation(**obs)
epos.set_survey(**survey) #)

''' Define a parametric population for quick comparison '''
epos.set_parametric(EPOS.fitfunctions.brokenpowerlaw2D)
epos.pdfpars.add('pps', 2.6)
epos.pdfpars.add('P break', 11., is2D=True)
epos.pdfpars.add('P1',1.6, is2D=True)
epos.pdfpars.add('P2',0.1, is2D=True)
epos.pdfpars.add('R break',3.4, is2D=True) 
epos.pdfpars.add('R1',-0.1, is2D=True)
epos.pdfpars.add('R2',-6.7, is2D=True)

''' Load a planet formation model '''
# NOTE: load your own planet formation / population synthesis model here

# generate some random data for testing EPOS
n= 616
sma= 10.**np.random.uniform(-1.3,1,n)
mass= 10.** (3.*np.random.power(0.5, n))
radius= 10.** np.random.power(0.5, n)
inc= np.random.rayleigh(2, n)
starID= np.repeat(np.arange(n/8), 8)

pfm= {'sma':sma, 'mass':mass,'radius':radius, 'inc':inc, 'starID':starID}

# Load the planet formaton model into EPOS
epos.set_population('Planet Formation Model', **pfm)
epos.fitpars.add('eta', 0.2, isnorm=True) # fraction of stars with simulated planets
epos.fitpars.add('f_iso', 0) # fraction of systems with isotropic orbits
epos.fitpars.add('f_inc', 1.0) # fudge factor for the inclinations
epos.fitpars.add('f_dP', 1.0) # fudge factor for the period ratios
epos.fitpars.add('f_cor', 0.5, fixed=True) # correlated noise

''' Uncomment to use a planet mass-radius conversion '''
#epos.set_massradius(EPOS.massradius.WRF15, 'WRF15+Rock', masslimits=[0.1,100])
#epos.set_massradius(EPOS.massradius.CK17, 'Chen & Kipping 2017', masslimits=[0.1,100])

''' Print some basic system properties'''
#print EPOS.analytics.pfmodel(epos)

''' define the simulated range (trim) and the range compared to observations (zoom)'''
epos.set_ranges(xtrim=[1,400],ytrim=[0.5,20.],xzoom=[1,400],yzoom=[0.5,20])

''' define rectangular occurrence rate bins'''
xbins= [[1,10], [20,300], [1, 20], [5,200]]
ybins= [[5,20], [4,15],   [0.7,2], [1.5,3.5]]
for xb, yb in zip(xbins, ybins):
	print 'area= {}'.format(np.log(yb[-1]/yb[0])*np.log(xb[-1]/xb[0]))
epos.set_bins(xbins=xbins,ybins=ybins)

''' Define polygonic occurrence rate bins'''
xmin, xmax= 1, 100
ymin, ymax= 0.7, 2.2
xb, yb= 3, 1.2
xyrock= [[xmin,ymin],[xmin,ymax],[xb,ymax],[xmax,yb],[xmax,ymin]]

xmin, xmax= 3, 200
ymin, ymax= 1.2, 4
xb, yb= 100, 2.2
xyMN= [[xmin,ymax],[xmax,ymax],[xmax,ymin], [xb,ymin], [xmin,yb]]

xyHJ= [[xbins[0][0],ybins[0][1]],[xbins[0][1],ybins[0][1]],
  [xbins[0][1],ybins[0][0]],[xbins[0][0],ybins[0][0]]]
xyTG= [[xbins[1][0],ybins[1][1]],[xbins[1][1],ybins[1][1]],
  [xbins[1][1],ybins[1][0]],[xbins[1][0],ybins[1][0]]]

epos.set_bins_poly([xyrock, xyMN, xyHJ, xyTG], 
  labels=['Hot\nEarths','Mini-\nNeptunes','Hot\nJupiters','Warm\nGiants'])

''' Run a forward model'''
EPOS.run.once(epos)

''' Calculate model occurrence rates '''
EPOS.occurrence.all(epos)

''' Save occurence rate to disk '''
EPOS.save.survey(epos)
EPOS.save.model(epos)
EPOS.save.synthetic_survey(epos)
EPOS.save.occurrence(epos)

''' Generate Plots '''
EPOS.plot.survey.all(epos)
EPOS.plot.input.all(epos, imin=1e-6, color='C8')
EPOS.plot.occurrence.all(epos, color='C8', alpha_fac=50.)
EPOS.plot.output.all(epos, color='C8')


#!/usr/bin/env ipython
from setuptools import setup, find_packages
from EPOS import __version__

exclude= [
  'EPOS/scriptdir/examples/example_4_massradius.py'
  'EPOS/scriptdir/examples/example_6_q16_catalog.py'
  'EPOS/scriptdir/examples/example_7_spectral_type.py'
  'EPOS/scriptdir/examples/example_8_no_monte_carlo.py'
  ] # also in MANIFEST.in

setup(name='epospy',
      version=__version__,
      description='the Exoplanet Population Observation Simulator',
      url='https://github.com/GijsMulders/epos',
      author='Gijs Mulders',
      author_email='gdmulders@gmail.com',
      license='MIT',
      packages=find_packages(exclude=exclude),
      install_requires=[
          'pytest >= 2.8',
          'numpy >= 1.13',
          'scipy',
          'matplotlib >= 2.0',
          'astropy',
          'emcee >= 2.0',
          'corner >= 2.0',
          'h5py',
	       'shapely'
      ],
      classifiers=[
        "Programming Language :: Python :: 2.7",
        "License :: OSI Approved :: MIT License",
      ],
      include_package_data=True,
#      zip_safe=False
)



#! /usr/bin/env ipython
''' 
Example script fitting a population of multi-planet systems to Kepler data
'''
import EPOS

''' initialize the EPOS class '''
epos= EPOS.epos(name='example_2')

''' load the kepler dr25 exoplanets and survey efficiency '''
obs, survey= EPOS.kepler.dr25(Huber=True, Vetting=True, score=0.9)
epos.set_observation(**obs)
epos.set_survey(**survey)

''' Define a double broken power-law as a planet population '''
epos.set_parametric(EPOS.fitfunctions.brokenpowerlaw2D)

''' Parameter initial guess and fitting ranges
Note: 
	- brokenpowerlaw2D uses 6 parameters, indicated with the is2D keyword
	- 'pps' is a normalization factor for the planet occurrence rate (planets-per-star)
	- parameter b_R is not fitted
	- dx is the range in walker initial positions for parameters that change sign (+/-)
'''
epos.fitpars.add('pps',		0.4, 	min=0)
epos.fitpars.add('P break',	10.,	min=2,	max=50,	is2D=True)
epos.fitpars.add('a_P',		1.5, 	min=0,			is2D=True)
epos.fitpars.add('b_P',		-1,		max=1,	dx=0.1,	is2D=True)
epos.fitpars.add('R break',	3.3,	fixed=True, 	is2D=True) 
epos.fitpars.add('a_R',		-0.5,	fixed=True, 	is2D=True)
epos.fitpars.add('b_R',		-6.,	fixed=True, 	is2D=True)

''' Set parameters for the spacing between planets'''
epos.set_multi(spacing='dimensionless')
epos.fitpars.add('npl', 10, fixed=True) # planets per system
epos.fitpars.add('log D', -0.3)
epos.fitpars.add('sigma', 0.2, min=0)

''' Some additional fit parameters'''
epos.fitpars.add('dR', 0.01, fixed=True)	# Dispersion in planet radii
epos.fitpars.add('inc', 2.0)				# mode of mutual inclinations
epos.fitpars.add('f_iso', 0.4)				# Fraction of isotropic systems
epos.fitpars.add('f_cor', 0.5, fixed=True)	# Correlated noise

''' define the simulated range (trim) and the range compared to observations (zoom)'''
epos.set_ranges(xtrim=[0,730],ytrim=[0.3,20.],xzoom=[2,400],yzoom=[1,6])

''' Run the Monte Carlo Simulation once '''
EPOS.run.once(epos)

''' run an MCMC chain on multiple cores or read in a previously saved run'''
EPOS.run.mcmc(epos, nMC=500, nwalkers=50, nburn=200, threads= 30, Saved=True) # ~20 mins
#EPOS.run.mcmc(epos, nMC=1000, nwalkers=100, nburn=200, threads=20, Saved=True) # ~5 hrs

''' Change the titles of the corner plots '''
epos.fitpars.latexkeys={
	'pps':'$\eta_s$', 
	'P break':'$P_{\\rm break}$', 'P1':'$a_P$', 'P2':'$b_P$',
	'P break':'$P_{\\rm break}$', 'R1':'$a_R$', 'R2':'$b_R$',
	'npl':'$n_p$', 'log D':'log $D$', 'sigma':'$\sigma$',
	'inc':'$i$', 'f_iso':'$f_{\\rm iso}$', 'f_cor':'$f_{\\rm cor}$'
}

''' plot everything '''
EPOS.plot.survey.all(epos)
EPOS.plot.input.all(epos)
EPOS.plot.output.all(epos)
EPOS.plot.mcmc.all(epos)
EPOS.plot.occurrence.all(epos)


import matplotlib.pyplot as plt
import numpy as np
import scipy.stats
from matplotlib import gridspec

import helpers
from EPOS.population import periodradius
from EPOS.fitfunctions import brokenpowerlaw1D

# Kepler obs
# Kepler pdf mass
# kepler pdf radius
# Kepler pdf inner (M/R ??)

fmt_symbol= {'ls':'', 'marker':'o', 'mew':1, 'mec':'k', 'ms':4,'alpha':0.3}

def panels_mass(epos, Population=False, color='C1'):
	f, (ax, axM, axP)= helpers.make_panels(plt)
	pfm=epos.pfm
	eta= epos.modelpars.get('eta',Init=True)

	''' Bins '''
	dw= 0.5 # bin width in ln space
	xbins= np.exp(np.arange(np.log(pfm['P limits'][0]),np.log(pfm['P limits'][-1])+dw,dw))
	ybins= np.exp(np.arange(np.log(pfm['M limits'][0]),np.log(pfm['M limits'][-1])+dw,dw))

	''' Posterior '''
	if Population:
		assert hasattr(epos, 'func')
		fname='.pop'
		
		# side panels marginalized over M and P limits
		pps, pdf, pdf_X, pdf_Y= periodradius(epos, Init=True)
		_, _, pdf_X, _= periodradius(epos, Init=True, ybin=ybins)
		_, _, _, pdf_Y= periodradius(epos, Init=True, xbin=xbins)
		pps, _ , _, _ = periodradius(epos, Init=True, xbin=xbins, ybin=ybins)
		#pdf/= np.max(pdf)
		#pdflog= np.log10(pdf) # in %
		levels= np.linspace(0,np.max(pdf))
		lines= np.array([0.1, 0.5]) * np.max(pdf)
		
		ax.contourf(epos.X_in, epos.Y_in, pdf, cmap='Purples', levels=levels)
		#ax.contour(epos.X_in, epos.Y_in, pdf, levels=lines)
		
		# Side panels
		#print 'pps model= {}'.format(eta)
		scale=dw
		axP.plot(epos.MC_xvar, pdf_X*scale, marker='',ls='-',color='purple')
		axM.plot(pdf_Y*scale, epos.in_yvar, marker='',ls='-',color='purple')
	else:
		fname=''

	''' plot main panel'''
	ax.set_title(epos.name)
	#helpers.set_axes(ax, epos, Trim=True)

	ax.set_xscale('log')
	ax.set_yscale('log')
	#ax.set_xlim(epos.mod_xlim)
	#ax.set_ylim(epos.mod_ylim)
		
	ax.plot(pfm['P'],pfm['M'], color=color, **fmt_symbol)
	xlim= ax.get_xlim()
	ylim= ax.get_ylim()
	
	''' Period side panel '''
	#axP.yaxis.tick_right()
	#axP.yaxis.set_ticks_position('both')
	#axP.tick_params(axis='y', which='minor',left='off',right='off')

	axP.set_xscale('log')
	axP.set_xlim(xlim)
	#ax.set_xlabel('Semi-Major Axis [au]')
	axP.set_xlabel('Orbital Period [days]')

	axP.hist(pfm['P'], color=color, bins=xbins, weights=np.full(pfm['np'], eta/pfm['np']) )

	''' Mass side panel'''
	#helpers.set_axis_size(axR, epos, Trim=True) #, In= epos.MassRadius)
	axM.set_ylabel(r'Planet Mass [M$_\bigoplus$]')
	axM.set_yscale('log')
	axM.set_ylim(ylim)

	axM.hist(pfm['M'], bins=ybins, orientation='horizontal', weights=np.full(pfm['np'], eta/pfm['np']), color=color )
	
	helpers.save(plt, '{}model/input.mass{}'.format(epos.plotdir, fname))
	
def panels_radius(epos, Population=False, Occurrence=False, Observation=False, 
	Tag=False, color='C0', clr_obs='C3', Shade=True, Fancy=True, Zoom=False):
	f, (ax, axR, axP)= helpers.make_panels(plt, Fancy=Fancy)
	pfm=epos.pfm
	eta= epos.modelpars.get('eta',Init=True)
	
	title=''
	if not 'R' in pfm:
		pfm['R'], _= epos.MR(pfm['M'])

	if Tag:
		# function that return a simulation subset based on the tag
		subset={'Fe/H<=0': lambda tag: tag<=0,
			'Fe/H>0': lambda tag: tag>0}

	''' Bins '''
	dwR=0.2 # bin width in ln space
	dwP=0.3
	if Zoom:
		xbins= np.exp(np.arange(np.log(epos.xzoom[0]),np.log(epos.xzoom[-1])+dwP,dwP))
		ybins= np.exp(np.arange(np.log(epos.yzoom[0]),np.log(epos.yzoom[-1])+dwR,dwR))	
	else:
		xbins= np.exp(np.arange(np.log(epos.xtrim[0]),np.log(epos.xtrim[-1])+dwP,dwP))
		ybins= np.exp(np.arange(np.log(epos.ytrim[0]),np.log(epos.ytrim[-1])+dwR,dwR))

	''' Plot model occurrence or observed counts'''
	if Observation:
		# plot model planets * completeness
		weights= eta *epos.occurrence['model']['completeness'] / pfm['ns']
	else:
		weights=np.full(pfm['np'], eta/pfm['ns'])
	
	if 'draw prob' in pfm and not Tag:
		prob= pfm['draw prob'][pfm['ID']] 
		weights*= prob*pfm['ns'] # system weights sum up to 1
		#nonzero= np.where(prob>0, 1., 0.)
		#weights*= nonzero*(pfm['np']/nonzero.sum())
	
	# histograms
	if Tag:
		for key, f in subset.iteritems():
			toplot= f(pfm['tag'])
			#weights= eta*epos.occurrence['model']['completeness'] \
			#		*np.where(toplot,1.,0.)/f(pfm['system tag']).sum()
			weights= np.where(toplot,eta,0.)/f(pfm['system tag']).sum()
			axP.hist(pfm['P'], bins=xbins, weights=weights, histtype='step', label=key)
			axR.hist(pfm['R'], bins=ybins, orientation='horizontal',
				weights=weights, histtype='step')
	else:
		# color have to be 1-element lists ??
		axP.hist(pfm['P'], bins=xbins, weights=weights, color=[color])
		axR.hist(pfm['R'], bins=ybins, orientation='horizontal', weights=weights, 
					color=[color])
				 
	''' Overplot observations? '''
	if Population:
		assert hasattr(epos, 'func')
		fname='.pop' +('.zoom' if Zoom else '')
		
		title= epos.title
		
		pps, pdf, pdf_X, pdf_Y= periodradius(epos, Init=True)
		_, _, pdf_X, _= periodradius(epos, Init=True, ybin=ybins)
		_, _, _, pdf_Y= periodradius(epos, Init=True, xbin=xbins)
		pps, _ , _, _ = periodradius(epos, Init=True, xbin=xbins, ybin=ybins)
		#pdf/= np.max(pdf)
		#pdflog= np.log10(pdf) # in %
		levels= np.linspace(0,np.max(pdf))
		lines= np.array([0.1, 0.5]) * np.max(pdf)
		
		if Shade:
			ax.contourf(epos.X_in, epos.Y_in, pdf, cmap='Purples', levels=levels)
			#ax.contour(epos.X_in, epos.Y_in, pdf, levels=lines)
		
			# Side panels
			#print 'pps model= {}'.format(eta)
			axP.plot(epos.MC_xvar, pdf_X*dwP, marker='',ls='-',color='purple')
			axR.plot(pdf_Y*dwR, epos.in_yvar, marker='',ls='-',color='purple')
		else:
			# renormalize
			xnorm= axP.get_ylim()[1]/max(pdf_X)
			ynorm= axR.get_xlim()[1]/max(pdf_Y)

			axP.plot(epos.MC_xvar, pdf_X*xnorm, marker='',ls='-',color=clr_obs)
			axR.plot(pdf_Y*ynorm, epos.in_yvar, marker='',ls='-',color=clr_obs)

	elif Observation:
		fname='.obs'+('.zoom' if Zoom else '')

		title= epos.title+': Counts'
		
		ax.plot(epos.obs_xvar, epos.obs_yvar, ls='', marker='.', ms=5.0, color='0.5')
		
		weights= np.full(epos.obs_xvar.size, 1./epos.nstars)
		axP.hist(epos.obs_xvar,bins=xbins,weights= weights, histtype='step', color='0.5')
		axR.hist(epos.obs_yvar,bins=ybins,weights= weights, 
			orientation='horizontal', histtype='step', color='0.5')
					
	elif Occurrence:
		fname='.occ'+('.zoom' if Zoom else '')
		title= epos.title+r': Occurrence, $\eta={:.2g}$'.format(eta)

		ax.plot(epos.obs_xvar, epos.obs_yvar, ls='', marker='.', ms=5.0, color='0.5')
		
		cut= epos.obs_yvar > 0.45
		
		weights= 1. / (epos.occurrence['planet']['completeness'][cut]*epos.nstars)
		axP.hist(epos.obs_xvar[cut],bins=xbins,weights=weights,histtype='step',color='k')
		axR.hist(epos.obs_yvar[cut],bins=ybins,weights= weights, 
			orientation='horizontal', histtype='step', color='k')
			
	elif Tag:
		fname='.tag'
		ax.set_title(epos.title+': Tag')
		
		axP.legend(frameon=False, fontsize='small')
		# 		for k, tag in enumerate(subset):
		# 			axP.text(0.98,0.95-0.05*k,tag,ha='right',va='top',color='C1', 
		# 				transform=axP.transAxes)
		
	else:
		fname=''

	if Fancy:
		plt.suptitle(title, ha='center')#, x=0.05)
	else:
		ax.set_title(title)

	''' plot main panel'''
	#helpers.set_axes(ax, epos, Trim=True)

	helpers.set_axes(ax, epos, Trim=True)
	if Tag:
		for key, f in subset.iteritems():
			todraw= f(pfm['tag'])
			ax.plot(pfm['P'][todraw],pfm['R'][todraw], **fmt_symbol)	
	elif 'draw prob' in pfm:
		#fmt_symbol['alpha']= 0.6*pfm['draw prob'][pfm['ID']] # alpha can't be array
		todraw= pfm['draw prob'][pfm['ID']]>0
		ax.plot(pfm['P'][todraw],pfm['R'][todraw], color=color, **fmt_symbol)
	else:
		ax.plot(pfm['P'],pfm['R'], color=color, **fmt_symbol)

	
	''' Period side panel '''
	#axP.yaxis.tick_right()
	#axP.yaxis.set_ticks_position('both')
	#axP.tick_params(axis='y', which='minor',left='off',right='off')
	helpers.set_axis_distance(axP, epos, Trim=True)
	
	''' Mass side panel'''
	helpers.set_axis_size(axR, epos, Trim=True) #, In= epos.MassRadius)
	
	helpers.save(plt, '{}model/input.radius{}'.format(epos.plotdir, fname))

def inclination(epos, color='C0', clr_obs='C3', imin=1e-2, Simple=False):
	pfm=epos.pfm
	
	gs = gridspec.GridSpec(1, 2,
                       width_ratios=[20,6],
                       )
	f= plt.figure()
	f.subplots_adjust(wspace=0)
	
	ax = plt.subplot(gs[0, 0])	
	axh = plt.subplot(gs[0, 1])
	axh.tick_params(direction='in', which='both', left=False, right=True, labelleft=False)
	axh.yaxis.set_label_position('right')
	axh.axis('off')

	''' Inc-sma'''	
	ax.set_title('Inclination {}'.format(epos.title))

	ax.set_xlabel('Semi-Major Axis [au]')
	#ax.set_ylabel(r'Planet Mass [M$_\bigoplus$]')
	ax.set_ylabel('Inclination [degree]')

	#ax.set_xlim(epos.mod_xlim)
	ax.set_ylim(imin,90)

	ax.set_xscale('log')
	ax.set_yscale('log')

	ax.axhline(np.median(pfm['inc']), ls='--', color=color)
	ax.plot(pfm['sma'], pfm['inc'], color=color, **fmt_symbol)

	''' Histogram'''
	axh.set_yscale('log')
	axh.set_ylim(imin,90)

	inc= np.logspace(np.log10(imin),2)
	axh.hist(pfm['inc'], bins=inc, orientation='horizontal', color=color) 
	
	#Model best-fit
	xmax= axh.get_xlim()[-1]
	if Simple:
		scale=2.
		pdf= scipy.stats.rayleigh(scale=scale).pdf(inc)
		pdf*= xmax/max(pdf)
		axh.plot(pdf, inc, ls='-', color=clr_obs)
		ax.axhline(scale, color=clr_obs, ls='--')	
	else:
		for scale, ls in zip([1,2,2.7],[':','--',':']):
			pdf= scipy.stats.rayleigh(scale=scale).pdf(inc)
			#pdf/= np.log(inc) #log scale?
			pdf*= xmax/max(pdf)
			axh.plot(pdf, inc, ls=ls, color=clr_obs)
			
			ax.axhline(scale, color=clr_obs, ls=ls)		

	helpers.save(plt, epos.plotdir+'model/inc-sma')
	
def periodratio(epos, color='C0', clr_obs='C3', Simple=False, Fancy=True):
	pfm=epos.pfm
	
	if Fancy:
		f, (ax, axR, axP)= helpers.make_panels(plt, Fancy=True)
		f.suptitle('Multi-planets {}'.format(epos.title))
	else:
		f, (ax, axR, axP)= helpers.make_panels_right(plt)
		ax.set_title('Input Multi-planets {}'.format(epos.title))


	''' Inc-sma'''	
	ax.set_xlabel('Semi-Major Axis [au]')
	ax.set_ylabel('Period ratio')

	#ax.set_xlim(epos.mod_xlim)
	ax.set_ylim(0.9,10)

	ax.set_xscale('log')
	ax.set_yscale('log')

	#ax.axhline(np.median(pfm['inc']), ls='--')
	single= pfm['dP'] == np.nan
	inner= pfm['dP'] == 1
	nth= pfm['dP']> 1
	
	# print pfm['dP'][single]
	# print pfm['dP'][nth]
	# print pfm['dP'][inner]
	# print pfm['dP'][nth].size, pfm['np'] # ok
	
	ax.plot(pfm['sma'][single], pfm['dP'][single], color='0.7', **fmt_symbol)
	ax.plot(pfm['sma'][nth], pfm['dP'][nth], color=color, **fmt_symbol)
	ax.plot(pfm['sma'][inner], pfm['dP'][inner], color='C1', **fmt_symbol)

	''' Histogram Period Ratio'''
	axR.set_yscale('log')
	axR.set_ylim(0.9,10)

	#dP= np.logspace(0,1,15)
	dP= np.logspace(0,1, 25)
	axR.hist(pfm['dP'][nth], bins=dP, orientation='horizontal', color=color)
	if not Simple:
		ax.axhline(np.median(pfm['dP'][nth]), ls='--', color=color, zorder=-1)
	
	#Model best-fit
	dP= np.logspace(0,1)
	xmax= axR.get_xlim()[-1]
	with np.errstate(divide='ignore'): 
		Dgrid= np.log10(2.*(dP**(2./3.)-1.)/(dP**(2./3.)+1.))
	Dgrid[0]= -2

	if Simple:
		scale= -0.37
		pdf= scipy.stats.norm(scale,0.19).pdf(Dgrid)
		pdf*= xmax/max(pdf)
		axR.plot(pdf, dP, ls='-', color=clr_obs)

		pscale= np.interp(scale,Dgrid,dP)
		#ax.axhline(pscale, color=clr_obs, ls='--')	
	else:
		for scale, ls in zip([-0.30,-0.37,-0.41],[':','--',':']):
			pdf= scipy.stats.norm(scale,0.19).pdf(Dgrid)
			pdf*= xmax/max(pdf)
			axR.plot(pdf, dP, ls=ls, color='purple')
			
			pscale= np.interp(scale,Dgrid,dP)
			ax.axhline(pscale, color='purple', ls=ls, zorder=-1)	

	''' Histogram Inner Planet'''
	axP.set_xscale('log')
	axP.set_xlim(ax.get_xlim())
	sma= np.geomspace(*ax.get_xlim(), num=25)
	axP.hist(pfm['sma'][inner], bins=sma, color='C1', label='Inner Planet')
	
	ymax= axP.get_ylim()[-1]
	P= np.geomspace(0.5,730)
	smaP= (P/365.25)**(1./1.5)
	pdf= brokenpowerlaw1D(P, 10,1.5,-0.8)
	pdf*= ymax/max(pdf)
	
	#axP.legend(frameon=False)
	if Fancy:
		ax.text(0.02,0.98,'Inner Planet',ha='left',va='top',color='C1', 
			transform=ax.transAxes)
		ax.text(0.02,0.98,'\nOuter Planet(s)',ha='left',va='top',color=color, 
			transform=ax.transAxes)
	else:
		axP.text(0.98,0.95,'Inner Planet',ha='right',va='top',color='C1', 
			transform=axP.transAxes)
	
	axP.plot(smaP, pdf, ls='-', color=clr_obs)

	
	#helpers.set_axes(ax, epos, Trim=True)
	#helpers.set_axis_distance(axP, epos, Trim=True)
	#helpers.set_axis_size(axR, epos, Trim=True) #, In= epos.MassRadius)

	helpers.save(plt, epos.plotdir+'model/Pratio-sma')

def periodratio_size(epos, color='C1'):
	pfm=epos.pfm
	
	f, (ax, axR, axP)= helpers.make_panels_right(plt)

	''' Inc-sma'''	
	ax.set_title('Input Multi-planets {}'.format(epos.name))

	axP.set_xlabel('Period ratio')
	#ax.set_ylabel(r'Size [$R_\bigoplus$]')

	ax.set_xlim(0.9,10)
	#ax.set_ylim(0.3,20)

	ax.set_xscale('log')
	#ax.set_yscale('log')

	helpers.set_axis_size(ax, epos, Trim=True)

	''' grids '''
	dP= np.logspace(0,1)
	dP_bins= np.logspace(0,1,15)

	# exoplanet data + hist
	ax.plot(epos.multi['Pratio'], epos.multi['Rpair'], ls='', marker='.', ms=5.0, color='0.5')

	#ax.axhline(np.median(pfm['inc']), ls='--')
	single= pfm['dP'] == np.nan
	inner= pfm['dP'] == 1
	nth= pfm['dP']> 1
	
	# print pfm['dP'][single]
	# print pfm['dP'][nth]
	# print pfm['dP'][inner]
	# print pfm['dP'][nth].size, pfm['np'] # ok
	
	ax.plot(pfm['dP'][single], pfm['R'][single], color='0.7', **fmt_symbol)
	ax.plot(pfm['dP'][nth], pfm['R'][nth], color=color, **fmt_symbol)
	ax.plot(pfm['dP'][inner], pfm['R'][inner], color='C1', **fmt_symbol)

	''' Histogram Period Ratio'''
	axP.set_xscale('log')
	axP.set_xlim(0.9,10)

	axP.hist(pfm['dP'][nth], bins=dP_bins, color=color)
	ax.axvline(np.median(pfm['dP'][nth]), ls='--', color=color)
	
	''' Model best-fit '''
	xmax= axP.get_ylim()[-1]
	with np.errstate(divide='ignore'): 
		Dgrid= np.log10(2.*(dP**(2./3.)-1.)/(dP**(2./3.)+1.))
	Dgrid[0]= -2

	for scale, ls in zip([-0.30,-0.37,-0.41],[':','--',':']):
		pdf= scipy.stats.norm(scale,0.19).pdf(Dgrid)
		pdf*= xmax/max(pdf)
		axP.plot(dP, pdf, ls=ls, color='purple')
		
		pscale= np.interp(scale,Dgrid,dP)
		ax.axvline(pscale, color='purple', ls=ls)	

	''' Raw data'''
	scale= 1.* pfm['dP'][nth].size/ epos.multi['Pratio'].size
	weights= np.full(epos.multi['Pratio'].size, scale)
	axP.hist(epos.multi['Pratio'], bins=dP_bins, weights=weights, histtype='step', color='0.5', zorder=1)

	''' Histogram Planet Size'''
	axR.set_yscale('log')
	axR.set_ylim(ax.get_ylim())
	radius= np.geomspace(*ax.get_ylim(), num=15)
	axR.hist(pfm['R'][inner], bins=radius, orientation='horizontal', color='C1', label='Inner Planet')
		
	#helpers.set_axes(ax, epos, Trim=True)
	#helpers.set_axis_distance(axP, epos, Trim=True)

	''' Linear regression '''
	try:
		# data
		slope, intercept, r_value, p_value, std_err= \
			scipy.stats.linregress(np.log(epos.multi['Pratio']), np.log(epos.multi['Rpair']))
		ax.plot(dP, np.exp(intercept+slope*np.log(dP)), label='r={:.2f}'.format(r_value), 
				marker='', ls='-', color='0.5')
		#print slope, intercept, r_value, p_value

		slope, intercept, r_value, p_value, std_err= \
			scipy.stats.linregress(np.log(pfm['dP'][nth]), np.log(pfm['R'][nth]))
		ax.plot(dP, np.exp(intercept+slope*np.log(dP)), label='r={:.2f}'.format(r_value), 
				marker='', ls='-', color=color)

		ax.legend(loc='lower right')
	except Exception as e: print(e)

	helpers.save(plt, epos.plotdir+'model/Pratio-size')

def periodratio_inc(epos, color='C1', imin=1e-2):
	pfm=epos.pfm
	
	f, (ax, axy, axx)= helpers.make_panels(plt, Fancy=True)

	''' Inc-sma'''	
	ax.set_title('Input Multi-planets {}'.format(epos.name))

	ax.set_xlabel('Period ratio')
	ax.set_ylabel('Inclination [degree]')

	ax.set_xlim(0.9,10)
	ax.set_ylim(imin,90)

	ax.set_xscale('log')
	ax.set_yscale('log')

	''' grids '''
	dP= np.logspace(0,1)
	dP_bins= np.logspace(0,1,15)

	#ax.axhline(np.median(pfm['inc']), ls='--')
	single= pfm['dP'] == np.nan
	inner= pfm['dP'] == 1
	nth= pfm['dP']> 1

	# exoplanet data + hist
	ax.plot(pfm['dP'][nth], epos.pfm['inc'][nth], color=color, **fmt_symbol)
	
	# print pfm['dP'][single]
	# print pfm['dP'][nth]
	# print pfm['dP'][inner]
	# print pfm['dP'][nth].size, pfm['np'] # ok
	
	#ax.plot(pfm['dP'][single], pfm['R'][single], color='0.7', **fmt_symbol)
	#ax.plot(pfm['dP'][nth], pfm['R'][nth], color=color, **fmt_symbol)
	#ax.plot(pfm['dP'][inner], pfm['R'][inner], color='C1', **fmt_symbol)

	''' Histogram Period Ratio'''
	axx.hist(pfm['dP'][nth], bins=dP_bins, color=color)
	ax.axvline(np.median(pfm['dP'][nth]), ls='--', color=color)

	ax.axvline(2, ls='--', color='b', zorder=0)
	ax.axvline(1.5, ls='--', color='b', zorder=0)

	''' Histogram Inclination'''
	#axy.set_yscale('log')
	#axy.set_ylim(ax.get_ylim())
	inc= np.logspace(np.log10(imin),2)
	axy.hist(pfm['inc'], bins=inc, orientation='horizontal', color=color) 
		
	#helpers.set_axes(ax, epos, Trim=True)
	#helpers.set_axis_distance(axP, epos, Trim=True)

	helpers.save(plt, epos.plotdir+'model/Pratio-inc')

def multiplicity(epos, color='C1', Planets=False, Kepler=False):
	# plot multiplicity
	f, ax = plt.subplots()
	ax.set_title('Input Multi-planets {}'.format(epos.name))
	ax.set_xlabel('planets per system')
	ax.set_ylabel('number of planets' if Planets else 'number of systems')

	if Kepler:
		ax.set_xlim(0.5, 7.5)	
	#else:
		#ax.set_xlim(0.5, 10.5)

	''' Model planets '''
	_ , counts= np.unique(epos.pfm['ID'],return_counts=True)
	#bins= np.arange(10)
	#ax.hist(counts, bins=bins, align='left', color=color)

	if Kepler:
		bins= np.arange(1,9)
		bins[-1]=1000
	else:
		bins= np.arange(20)
	hist, bin_edges = np.histogram(counts, bins=bins)
	ax.bar(bins[:-1], hist, width=1, color=color)

	''' Kepler '''
	# intrinsic= np.zeros_like(hist)
	# intrinsic[0]= 0.5*epos.pfm['ns']
	# intrinsic[-1]= 0.5*epos.pfm['ns']
	# #intrinsic=np.arange(len(hist))
	# ax.bar(bins[:-1], intrinsic, width=1, color='', ec='k')
	
	#ax.plot(, drawstyle='steps-mid', 
	#	ls='--', marker='', color='gray', label='Kepler all')

	ax.legend(loc='upper right', shadow=False, prop={'size':14}, numpoints=1)

	helpers.save(plt, epos.plotdir+'model/multi')

def period(epos, Population=False, Occurrence=False, Observation=False, Tag=False, 
	color='C1', Zoom=False, Rbin=[1.,4.]):
	''' Model occurrence as function of orbital period'''
	f, ax = plt.subplots()
	helpers.set_axis_distance(ax, epos, Trim=True)
	ax.set_xlim(0.5,200)
	
	ax.set_ylabel(r'Planet Occurrence {:.2g}-{:.2g} $R_\bigoplus$'.format(*Rbin))
	ax.set_yscale('log')
	
	pfm=epos.pfm
	eta= epos.modelpars.get('eta',Init=True)
	
	if not 'R' in pfm:
		pfm['R'], _= epos.MR(pfm['M'])

	if Tag:
		# function that return a simulation subset based on the tag
		subset={'Fe/H<=0': lambda tag: tag<=0,
			'Fe/H>0': lambda tag: tag>0}

	''' Bins '''
	#xbins= np.geomspace(1,1000,10)/(10.**(1./3.))
	#xbins= np.geomspace(0.5,200,15)

	dwP=0.3 # bin width in ln space
	
	if Zoom:
		xbins= np.exp(np.arange(np.log(epos.xzoom[0]),np.log(epos.xzoom[-1])+dwP,dwP))
	else:
		xbins= np.exp(np.arange(np.log(epos.xtrim[0]),np.log(epos.xtrim[-1])+dwP,dwP))

	''' Plot model occurrence or observed counts'''
	weights=np.full(pfm['np'], eta/pfm['ns'])
	
	if 'draw prob' in pfm and not Tag:
		prob= pfm['draw prob'][pfm['ID']] 
		weights*= prob*pfm['ns'] # system weights sum up to 1
	
	# histograms
	if Tag:
		for key, f in subset.iteritems():
			toplot= f(pfm['tag']) #& (pfm['R']>1.)
			weights= np.where(toplot,eta,0.)/f(pfm['system tag']).sum()			

			ax.hist(pfm['P'], bins=xbins, weights=weights, histtype='step', label=key,
				color='#88CCEE' if key=='Fe/H<=0' else '#332288')
	else:
		ax.hist(pfm['P'], bins=xbins, weights=weights, color=color, histtype='step')
				 
			
	if Tag:
		fname='.tag'
		ax.set_title(epos.name+': Disk Fe/H')
		ax.legend(fontsize='small', loc='lower right')		
	else:
		fname=''	

	if Occurrence:
		cut= (Rbin[0]<epos.obs_yvar) & (epos.obs_yvar<Rbin[-1])
		
		weights= epos.occurrence['planet']['occ'][cut]
		ax.hist(epos.obs_xvar[cut],bins=xbins,weights=weights,histtype='step',color='k')
		
		helpers.save(plt, '{}occurrence/model.period{}'.format(epos.plotdir, fname))

	# elif Observation: # counts
	else:
		helpers.save(plt, '{}model/input.period{}'.format(epos.plotdir, fname))

def HansenMurray(epos, color='purple'):
	''' figures not included '''
	import matplotlib.image as mpimg

	pfm= epos.pfm
	print pfm.keys()

	''' Hansen & Murray 2013 Figure 1 '''
	f, ax = plt.subplots()
	ax.set_xlabel('$N_p$')
	ax.set_ylabel('counts')
	ax.set_xlim([0.5,10.5])
	ax.set_ylim([0,35])
	
	fig1 = mpimg.imread('HM13figs/fig1_cut.png')
	ax.imshow(fig1, aspect='auto', extent=[0.5,10.5,0,35])

	_ , counts= np.unique(pfm['ID'],return_counts=True)
	ax.hist(counts, bins=np.arange(0,11), align='left', 
		color='purple', alpha=0.7, weights=np.full_like(counts, 2) )
	
	helpers.save(plt, '{}HM13/fig1'.format(epos.plotdir), dpi=300)

	''' Hansen & Murray 2013 Figure 5 '''
	f, ax = plt.subplots()
	ax.set_xlabel('$P_2/P_1$')
	ax.set_ylabel('counts')
	ax.set_xlim([1,4])
	ax.set_ylim([0,32])
	#ax.set_xscale('log')
	#ax.set_yscale('log')
	
	fig1 = mpimg.imread('HM13figs/fig5_cut.png')
	ax.imshow(fig1, aspect='auto', extent=[1,4,0,32])

	dP= pfm['dP'][pfm['dP']>1]
	ax.hist(dP, bins=np.linspace(1.+(1.5/37.),4,38), align='mid', 
		color='purple', alpha=0.7, weights=np.full_like(dP, 2))
	
	helpers.save(plt, '{}HM13/fig5'.format(epos.plotdir), dpi=300)


	''' Hansen & Murray 2013 Figure 9 '''
	f, ax = plt.subplots()
	ax.set_xlabel('Eccentricity')
	ax.set_ylabel('counts')
	ax.set_xlim([0,0.24])
	ax.set_ylim([0,35])
	#ax.set_xscale('log')
	#ax.set_yscale('log')
	
	fig1 = mpimg.imread('HM13figs/fig9_cut.png')
	ax.imshow(fig1, aspect='auto', extent=[0,0.24,0,35])

	ax.hist(pfm['ecc'], bins=np.linspace(0.01,0.25,25), align='left', 
		color='purple', alpha=0.7, weights=np.full_like(pfm['ecc'], 2))
	
	helpers.save(plt, '{}HM13/fig9'.format(epos.plotdir), dpi=300)

	''' Hansen & Murray 2013 Figure 10 '''
	f, ax = plt.subplots()
	ax.set_xlabel('sma [au]')
	ax.set_ylabel('Inclination')
	ax.set_xlim([0.04,1.2])
	ax.set_ylim([0,33])
	#ax.set_xscale('log')
	
	fig10 = mpimg.imread('HM13figs/fig10_cut.png')
	ax.imshow(fig10, aspect='auto', extent=[0.04,1.2,0,33])
	ax.get_xaxis().set_ticks([])

	axlog=ax.twiny()
	axlog.set_xlim([0.04,1.2])
	axlog.set_xscale('log')
	axlog.plot(pfm['sma'], pfm['inc'], color='purple', alpha=0.7, 
		marker='o', ls='')
	
	helpers.save(plt, '{}HM13/fig10'.format(epos.plotdir), dpi=300)

	''' Hansen & Murray 2013 Figure 11 '''
	f, (axa, axb) = plt.subplots(2)
	fig11a = mpimg.imread('HM13figs/fig11a_cut.png')
	fig11b = mpimg.imread('HM13figs/fig11b_cut.png')

	#ax[0].set_xlabel('Inclination')
	axa.set_ylabel('counts')
	axa.set_xlim([0,24.5])
	axa.set_ylim([0,20])

	axb.set_xlabel('Inclination')
	axb.set_ylabel('counts')
	axb.set_xlim([0,14.5])
	axb.set_ylim([0,35])
	
	axa.imshow(fig11a, aspect='auto', extent=[0,24.5,0,20])
	axb.imshow(fig11b, aspect='auto', extent=[0,14.5,0,35])

	inner= pfm['inc'][pfm['sma']<0.1]
	outer= pfm['inc'][(pfm['sma']>0.1) & (pfm['sma']<1.0)]

	axa.hist(inner, bins=np.linspace(0,25,26), align='mid', 
		color='purple', alpha=0.7, weights=np.full_like(inner, 2))
	axb.hist(outer, bins=np.linspace(0,15,50), align='mid', 
		color='purple', alpha=0.7, weights=np.full_like(outer, 2))
	
	helpers.save(plt, '{}HM13/fig11'.format(epos.plotdir), dpi=300)



import os.path, shutil
import glob

# is there a less clumsy way to do this?
import EPOS
fpath= os.path.dirname(EPOS.__file__)

def install(fdir='epos-scripts'):
	'''
	Installs the tests and examples to the directory epos-scripts/
	'''
	#if not os.path.isdir(fdir): os.makedirs(fdir)

	for subdir in ['tests','examples','papers']:
		scriptdir= '{}/{}'.format(fdir, subdir)
		if not os.path.isdir(scriptdir): os.makedirs(scriptdir)

		print '\nCopying scripts into {}'.format(scriptdir)

		flist= glob.glob('{}/scriptdir/{}/*.py'.format(fpath, subdir))
		for fname in flist:
			fname_stripped= fname.split('/')[-1]
			fdest= '{}/{}'.format(scriptdir, fname_stripped)
			shutil.copyfile(fname, fdest)
			# make them executable
			print '  copied {}'.format(fname_stripped)

	print '\nRun scripts with\n'
	print '  ./test_1_survey.py\n'
	print 'or\n'
	print '  ipython test_1_survey.py\n'


import h5py
import glob
import numpy as np
import sys, os

import cgs

''' Helper functions to read in planet formation models'''

#def symba(name='HMSim1', dir='hdf5/Sim1', plts_mass=0, istep=None, Verbose=False):
def symba(name, fname, plts_mass=0, cut=-np.inf, smacut=np.inf, istep=None, 
	Verbose=False, Saved=True):
	''' 
	returns a list of planetary systems
	sma in au
	mass in earth masses
	remove planetesimals (m < plt_mass)
	cut planets that are still forming (m < cut*au)
	'''
	
	dir= 'npz/{}'.format(name)
	if not os.path.exists(dir): os.makedirs(dir)
	fnpz= '{}/{}.npz'.format(dir, name)	
	
	''' Load hdf5 file or npz dictionary for quicker access'''
	if os.path.isfile(fnpz) and Saved:
		print '\nLoading saved status from {}'.format(fnpz)
		npz= np.load(fnpz)
				
		# check if keys present
		for key in ['sma','mass','inc','starID']:
			if not key in npz: 
				raise ValueError('Key {} not present\n{}'.format(key,npz.keys()))
	else:
		print '\nProcessing Symba HDF5 file for {}'.format(name)
		#fname= '{}/{}_set??.h5'.format(dir,name)
		flist= glob.glob(fname)
		if len(flist)==0: 
			raise ValueError('file not found: {}'.format(fname))
		else:
			if Verbose: print '  {} files'.format(len(flist))
	
		sma, mass, inc, ecc, ID= [], [], [], [], []
		sma0, mass0, inc0, ecc0, ID0 =[], [], [], [], []
		symbamassunit= cgs.Msun/cgs.Mearth / (2.*np.pi)**2.
	
		for i,fname in enumerate(flist):
			with h5py.File(fname,'r') as hf:

				# get final steps
				if istep is None:
					nsteps=0
					for particle in hf:
						nsteps=max(nsteps, hf.get(particle).shape[0])
				else:
					nsteps=istep

				if Verbose: 
					print '\n{}, step {}'.format(fname, nsteps)
				else:
					amtDone= float(i)/len(flist)
					print '\r  [{:50s}] {:5.1f}%'.format('#' * int(amtDone * 50), amtDone * 100),
					sys.stdout.flush() 
			
				L_sma, L_mass, L_inc, L_ecc= [], [], [], []
				L_sma0, L_mass0, L_inc0, L_ecc0= [], [], [], []
			
				for particle in hf:
					#if Verbose: print particle, hf.get(particle).shape[0]
					if hf.get(particle).shape[0] == nsteps:
						final_architecture= np.array(hf.get(particle))[-1,:]

						_mass= final_architecture[8]* symbamassunit
						_sma= final_architecture[2]
						if (_mass>plts_mass) & (_mass>cut*_sma**1.5) & (_sma<smacut):
							L_sma.append(_sma)
							L_mass.append(_mass)
							L_inc.append(final_architecture[4])
							L_ecc.append(final_architecture[3])
							ID.append(i)

					# initial conditions
					if hf.get(particle).shape[0]== 1:
						pass
					else:
						#print particle
						initial_architecture= np.array(hf.get(particle))[0,:]
						L_sma0.append(initial_architecture[2])
						L_mass0.append(initial_architecture[8]* symbamassunit)
						L_inc0.append(initial_architecture[4])
						L_ecc0.append(initial_architecture[3])
						ID0.append(i)

						#if not 'time' in sg: sg['time']= final_architecture[0]/1e6
				
					# 			# print '\nLoaded subgroup {} with {} planetary systems'.format(sg['name'], sg['n'])
	# 			for system in sg['system']:
	# 				print 'system has {} planets, {:.1f} Mearth:'.format(
	# 					system['np'],np.sum(system['mass']))
	# 				for a,m in zip(system['sma'], system['mass']):
	# 					print '  {:.2f} au, {:.1f} Mearth'.format(a,m)
					#print sg['all_Pratio']

				
					#initial_condition= np.array(hf.get(particle))[0,:]
					#print hf.get(particle).shape
					#L_sma0.append(initial_condition[2]) ?? crashes at HMSim3
					#L_mass0.append(initial_condition[8]* symbamassunit)
				
					# -Col 1: time in yrs
					# -Col 2: Body identifier
					# -Col 3: Semi major axis
					# -Col 4: Eccentricity
					# -Col 5: Inclination
					# -Col 6: Longitude of ascending node
					# -Col 7: Argument of Periapsis
					# -Col 8: Mean anomaly at epoch
					# -Col 9: Mass in units where 1 solar mass = (2*pi)^2
			
				# print system properties:
				if Verbose:
					print 'System {} has {} planets, {:.1f} Mearth:'.format(
						i, len(L_sma),np.sum(L_mass))
					for a,m in zip(L_sma, L_mass):
						print '  {:.2f} au, {:.1f} Mearth'.format(a,m)
			
				sma.extend(L_sma)
				mass.extend(L_mass)
				inc.extend(L_inc)
				ecc.extend(L_ecc)
				#ID (set in loop)
			
				sma0.extend(L_sma0)
				mass0.extend(L_mass0)
				inc0.extend(L_inc0)
				ecc0.extend(L_ecc0)

			if not Verbose: print '\r  [{:50s}] {:.1f}%'.format('#' * int(1 * 50), 1 * 100),
		
		npz={'sma':np.asarray(sma), 'mass':np.asarray(mass), 
			'inc':np.asarray(inc), 'starID':np.asarray(ID),
			'ecc':np.asarray(ecc),
			'sma0':np.asarray(sma0), 'mass0':np.asarray(mass0), 
			'inc0':np.asarray(inc0), 'starID0':np.asarray(ID0),
			'ecc0':np.asarray(ecc0)}
		
		#if Saved:
		print '\nSaving status in {}'.format(fnpz)
		#np.save(fname, epos.chain)
		# compression slow on loading?
		np.savez_compressed(fnpz, **npz)
		
	return npz
	
def mercury(fname, istep=None, icut=-np.inf, smacut=[0,np.inf], masscut= [0,np.inf],
	Verbose=False):
	print '\nProcessing Mercury file'
	flist= glob.glob(fname)
	if len(flist)==0: raise ValueError('file not found: {}'.format(fname))
	sma, mass, inc, ID= [], [], [], []
	
	for i,fname in enumerate(flist):

		if Verbose: 
			print '  {}: Using planetary system at final step'.format(fname)
		else:
			amtDone= float(i)/len(flist)
			print '\r  [{:50s}] {:5.1f}%'.format('#' * int(amtDone * 50), amtDone * 100),
			sys.stdout.flush() 
		
		try:
			a= np.loadtxt(fname, unpack=True,ndmin=2) # ndmin to guarantee lists

			L_sma= a[1]
			L_mass= a[7] * cgs.Msun/cgs.Mearth
			L_inc= a[3]
		
			# story copy(?) in list of planetary systems
			# cut out systems with low inclinatons (optional)
			if np.median(L_inc) > icut:
				sma.extend(L_sma)
				mass.extend(L_mass)
				inc.extend(L_inc)
				ID.extend([i]*len(L_sma))

		except ValueError:
			print '\n(skipping {})\n'.format(fname)

		except: raise

		if not Verbose: print '\r  [{:50s}] {:.1f}%'.format('#' * int(1 * 50), 1 * 100),

	sma= np.array(sma)
	mass=np.array(mass)
	inc=np.array(inc)
	starID=np.array(ID)

	# cut out certain planets
	include= (smacut[0]<=sma) & (sma<=smacut[-1])
	include&= (masscut[0]<=mass) & (mass<=masscut[-1])

	npz={'sma':sma[include], 'mass':mass[include], 
		'inc':inc[include], 'starID':starID[include]}
		
	return npz

def pa_bert(name='1Dlin', dir='PA_Bert/', Verbose=False):
	fname= '{}/Data{}.out'.format(dir,name)
	#with open(fanem,'r') as f:
	a= np.loadtxt(fname, unpack=True)
	include= np.isfinite(a[4]) & (a[4]>0) # & (a[-1]<-0.2) # metallicity cut
	sma= a[4][include]
	mass= a[5][include]
	FeH= a[-1][include]
	
	print '\nLoad Pebble Accretion from Bertram Bitsch'
	print '  sma:  {:.2e} ... {:.1f}'.format(min(sma), max(sma))
	print '  mass: {:.2e} ... {:.1f}'.format(min(mass), max(mass))
	print '  Fe/H: {:.2f} ... {:.2f}'.format(min(FeH), max(FeH))
	
	return sma, mass, FeH
	
def bern(name='syntheticpop_20emb_983systems.txt', dir='Bern', 
		smacut=[0,np.inf], masscut= [0,np.inf], Rcut=[0,np.inf], 
		Verbose=False, Single=False):
	fname= '{}/{}'.format(dir,name)

	''' Read the header '''
	header= np.genfromtxt(fname, max_rows=1, dtype=str, delimiter=',')
	if Verbose:
		#print header
		print '\nColumns:'
		for k, colname in enumerate(header):
			print '  a[{}]: {}'.format(k, colname)
	#print k

	''' Read the data '''
	a= np.loadtxt(fname, unpack=True, skiprows=1)
	if Verbose:
		print '\nraw data: {} systems, {} planets'.format(np.unique(
			a[0 if Single else 1]).size, a[0].size)

	# cut out certain planets
	include= (smacut[0]<=a[2]) & (a[2]<=smacut[-1])
	include&= (masscut[0]<=a[3]) & (a[3]<=masscut[-1])
	include&= (Rcut[0]<=a[4]) & (a[4]<=Rcut[-1])
	#include&= (cut[0]<=a[]) & (a[]<=cut[-1])

	#planet= a[0][include]
	ID= a[0 if Single else 1][include]
	sma= a[2][include]
	mass= a[3][include]
	radius= a[4][include]
	ecc= a[5][include]
	inc=a[6][include]
	FeH= a[7][include]
	Mcore= a[8][include]
	Menv= a[9][include]
	sma0= a[10][include]
	fice= a[11][include]

	print '\nLoad population synthesis model {}'.format(name)
	print '  included {} systems, {} planets'.format(np.unique(ID).size, sma.size)
	print '  sma:  	 {:.2e} ... {:.1f}'.format(min(sma), max(sma))
	print '  sma0: 	 {:.2e} ... {:.1f}'.format(min(sma0), max(sma0))
	print '  mass:   {:.2e} ... {:.1f}'.format(min(mass), max(mass))
	print '  radius: {:.2f} ... {:.1f}'.format(min(radius), max(radius))
	print '  inc:    {:.2e} ... {:.1f}'.format(min(inc), max(inc))
	print '  Fe/H:   {:.2f} ... {:.2f}'.format(min(FeH), max(FeH))
	
	order= np.lexsort((sma,ID)) 
	
	npz={'sma':sma[order], 'mass':mass[order], 'radius':radius[order], 
		'inc':inc[order], 'ecc':ecc[order], 'starID':ID[order], 
		'Mcore':Mcore[order], 'Menv':Menv[order], 'fice':fice[order],
		'tag':FeH[order], 'sma0':sma0[order]}
	
	if Single: npz['inc']=None

	return npz

def morby(name, fname, Verbose=False, Saved=True):
	''' 
	returns a list of planetary systems
	sma in au
	mass in earth masses
	'''
	
	dir= 'npz/{}'.format(name)
	if not os.path.exists(dir): os.makedirs(dir)
	fnpz= '{}/{}.npz'.format(dir, name)	
	
	''' Load hdf5 file or npz dictionary for quicker access'''
	if os.path.isfile(fnpz) and Saved:
		print '\nLoading saved status from {}'.format(fnpz)
		npz= np.load(fnpz)
				
		# check if keys present
		for key in ['sma','mass','inc','starID']:
			if not key in npz: 
				raise ValueError('Key {} not present\n{}'.format(key,npz.keys()))
	else:
		print '\nProcessing Symba HDF5 file for {}'.format(name)
		#fname= '{}/{}_set??.h5'.format(dir,name)
		flist= glob.glob(fname)
		if len(flist)==0: 
			raise ValueError('file not found: {}'.format(fname))
		else:
			if Verbose: print '  {} files'.format(len(flist))
	
		sma, mass, inc, ecc, ID= [], [], [], [], []
	
		for i,fname in enumerate(flist):
			a= np.loadtxt(fname,skiprows=1, usecols=(1,2,3,7), unpack=True)
			#if a.ndim==1: a=[a]
			try:
				if a.ndim==1:
					sma.append([a[0]])
					ecc.append([a[1]])
					inc.append([a[2]])
					mass.append([a[3]])

					ID.append([i])
				elif a.ndim==2:

					sma.append(a[0])
					ecc.append(a[1])
					inc.append(a[2])
					mass.append(a[3])

					ID.append([i]*len(a[0]))
				else:
					print a.shape
					print a
					raise ValueError('Error reading in') 
			except (IndexError, TypeError):
				print a.shape
				print a
				raise

			# print system properties:
			if Verbose:
				print 'System {} has {} planets, {:.1f} Mearth:'.format(
					i, len(sma[-1]),np.sum(mass[-1]))
				for items in zip(sma[-1], mass[-1], inc[-1]):
					print '  {:.2f} au, {:.1f} Mearth, {:.3g} deg'.format(*items)
		
		npz={'sma':np.concatenate(sma), 'mass':np.concatenate(mass), 
			'inc':np.concatenate(inc), 'starID':np.concatenate(ID),
			'ecc':np.concatenate(ecc)}
		
		if Saved:
			print 'Saving status in {}'.format(fnpz)
			#np.save(fname, epos.chain)
			# compression slow on loading?
			np.savez_compressed(fnpz, **npz)
		
	return npz


def mordasini(name='syntheticpopmordasini1MsunJ31', dir='Mordasini', smacut=np.inf,
		Rcut=0, Single=False, Verbose=False):
	fname= '{}/{}.dat'.format(dir,name)
	header= np.genfromtxt(fname, max_rows=1, dtype=str)
	print header
	a= np.loadtxt(fname, unpack=True, skiprows=1, usecols=(0,1,2,3,4,6,7))

	npl= a[2].size
	ns= np.unique(a[0 if Single else 1]).size

	include= (a[2]<smacut) & (a[4]>Rcut)
	ID= a[0 if Single else 1][include]
	sma= a[2][include]
	mass= a[3][include]
	radius= a[4][include]
	inc=a[5][include]
	FeH= a[6][include]

	print '\nLoad population synthesis model {}'.format(name)
	print '  {} stars, {} planets'.format(ns, npl)
	print '  sma:  	 {:.2e} ... {:.1f}'.format(min(sma), max(sma))
	print '  mass:   {:.2e} ... {:.1f}'.format(min(mass), max(mass))
	print '  radius: {:.2f} ... {:.1f}'.format(min(radius), max(radius))
	print '  inc:    {:.2e} ... {:.1f}'.format(min(inc), max(inc))
	print '  Fe/H:   {:.2f} ... {:.2f}'.format(min(FeH), max(FeH))
	
	order= np.lexsort((sma,ID)) 
	
	npz={'sma':sma[order], 'mass':mass[order], 'radius':radius[order], 
		'inc':inc[order], 'starID':ID[order], 'tag':FeH[order]}

	if Single: npz['inc']=None
		
	return npz

def mordasini_ext(name='syntheticpopmordasini1MsunJ31extended', dir='Mordasini', smacut=np.inf,
		Rcut=0, Verbose=False):
	fname= '{}/{}.dat'.format(dir,name)
	header= np.genfromtxt(fname, max_rows=1, dtype=str)
	print header
	a= np.loadtxt(fname, unpack=True, skiprows=1, usecols=(1,2,3,4,6,7,10))

	npl= a[2].size
	ns= np.unique(a[0 if Single else 1])

	include= (a[1]<smacut) & (a[3]>Rcut)
	ID= a[0][include]
	sma= a[1][include]
	mass= a[2][include]
	radius= a[3][include]
	inc=a[4][include]
	FeH= a[5][include]
	sma0= a[6][include]

	print '\nLoad population synthesis model {}'.format(name)
	print '  {} stars, {} planets'.format(ns, npl)
	print '  sma:  	 {:.2e} ... {:.1f}'.format(min(sma), max(sma))
	print '  sma0: 	 {:.2e} ... {:.1f}'.format(min(sma0), max(sma0))
	print '  mass:   {:.2e} ... {:.1f}'.format(min(mass), max(mass))
	print '  radius: {:.2f} ... {:.1f}'.format(min(radius), max(radius))
	print '  inc:    {:.2e} ... {:.1f}'.format(min(inc), max(inc))
	print '  Fe/H:   {:.2f} ... {:.2f}'.format(min(FeH), max(FeH))
	
	order= np.lexsort((sma,ID)) 
	
	npz={'sma':sma[order], 'mass':mass[order], 'radius':radius[order], 
		'inc':inc[order], 'starID':ID[order], 'tag':FeH[order], 'sma0':sma0[order]}
		
	return npz

def combine(pfmlist, tags):
	''' Combine Multiple Planet Formation Model'''
	assert len(tags) == len(pfmlist)

	''' Create list of tags '''
	taglist= []
	for pfm, tag in zip(pfmlist, tags):
		taglist.append(np.full_like(pfm['sma'], tag))

	''' Create merged dictionary'''
	npz={'tag':np.concatenate(taglist)}
	for key in pfm.keys():
		npz[key]= np.concatenate([pfm[key] for pfm in pfmlist])

	return npz


import matplotlib, warnings
#warnings.filterwarnings("ignore", category=UserWarning, module='matplotlib')
with warnings.catch_warnings():
	warnings.simplefilter("ignore")
	matplotlib.use('Agg')

import matplotlib.pyplot as plt
import matplotlib.patches as patches
import numpy as np

import parametric, multi, massradius, helpers, model

def all(epos, color=None, imin=1e-2):
	print '\nPlotting input...'
	
	#add decorators for zoom T/F?
	# color into epos?

	if epos.Parametric:
		parametric.oneD(epos)
		parametric.twoD(epos)
		parametric.panels(epos)

		# plot the input distribution converted from M->Msini or M->R
		if not epos.MonteCarlo and epos.Msini:
			parametric.oneD_y(epos, Convert=True) # no conversion for x (yet)
			parametric.twoD(epos, Convert=True)
			parametric.panels(epos, Convert=True)

	else:
		if 'R' in epos.pfm:
			model.panels_radius(epos, color=color)		
		if 'M' in epos.pfm:
			model.panels_mass(epos, color=color)
		
		model.period(epos, color=color)
			
		model.multiplicity(epos, color=color)
		if hasattr(epos, 'func'):
			if epos.MassRadius:
				model.panels_mass(epos, Population=True, color=color)
			else:
				model.panels_radius(epos, Population=True, Shade=False, color=color) #Zoom?

		if hasattr(epos, 'occurrence'):
			# model counts (debiases data)
			if 'planet' in epos.occurrence:
				model.panels_radius(epos, Occurrence=True, color=color)
				if 'R' in epos.pfm or epos.MassRadius:
					model.period(epos, Occurrence=True, color=color)
				if epos.Zoom:
					model.panels_radius(epos, Occurrence=True, color=color, Zoom=True)

			# planet counts (biases model)
			if 'model' in epos.occurrence:
				model.panels_radius(epos, Observation=True, color=color)
				if epos.Zoom:
					model.panels_radius(epos, Observation=True, color=color, Zoom=True)

		if 'tag' in epos.pfm:
			model.panels_radius(epos, Tag=True) # Zoom?
			model.period(epos, Tag=True)
			
		if 'inc' in epos.pfm:
			model.inclination(epos, Simple=True, color=color, imin=imin)
		if 'dP' in epos.pfm:
			model.periodratio(epos, color=color, Simple=True)
			if 'R' in epos.pfm:
				model.periodratio_size(epos, color=color)
			if 'inc' in epos.pfm:
				model.periodratio_inc(epos, color=color, imin=imin)

	if epos.Multi:
		if epos.Parametric:
			multi.periodradius(epos, MC=False)
			multi.periodradius(epos, MC=False, Nth=True)
			multi.multiplicity(epos, MC=False)
			multi.periodratio(epos, MC=False)
			multi.periodratio_cdf(epos, MC=False)
			multi.periodinner(epos, MC=False)
			multi.periodinner_cdf(epos, MC=False)
		else:
			pass
		
	if epos.MassRadius: 
		massradius.massradius(epos, MC=False)
		massradius.massradius(epos, MC=False, Log=True)



